@@ -13,8 +13,12 @@
 #include <openssl/rsa.h>
 #include <openssl/sha.h>
 #include <openssl/md5.h>
-#include <openssl/x509.h>
 #include <openssl/ssl.h>
+#include <openssl/rand.h>
+#include <openssl/asn1.h>
+#include <openssl/x509.h>
+#include <openssl/x509_vfy.h>
+#include <openssl/x509v3.h>
 
 #if __APPLE__
 #if defined MAC_OS_X_VERSION_10_7 && MAC_OS_X_VERSION_MIN_REQUIRED >= 1070
